unit UMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, MidiIn, MidiOut, MidiType, unitMidiGlobals, Grids, UGlob,
  Gauges;

type
  TMainForm = class(TForm)
    GroupBox3: TGroupBox;
    btCut: TButton;
    btCapture: TButton;
    btSend: TButton;
    btEdit: TButton;
    MidiOutputPort: TMidiOutput;
    MidiInputPort: TMidiInput;
    btCopy: TButton;
    btPaste: TButton;
    GroupBox1: TGroupBox;
    tbSysExList: TStringGrid;
    btName: TButton;
    btAuto: TButton;
    btSendAll: TButton;
    btFactory: TButton;
    btImport: TButton;
    OpenDialog: TOpenDialog;
    gProgress: TGauge;
    btExport: TButton;
    SaveDialog: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure btCaptureClick(Sender: TObject);
    procedure MidiInputPortMidiInput(Sender: TObject);
    procedure btNameClick(Sender: TObject);
    procedure btAutoClick(Sender: TObject);
    procedure btEditClick(Sender: TObject);
    procedure btSendClick(Sender: TObject);
    procedure DeleteRow(RowNr : Byte);
    procedure UpdateRow(RowNr : Byte);    
    procedure btCutClick(Sender: TObject);
    procedure btSendAllClick(Sender: TObject);
    procedure btPasteClick(Sender: TObject);
    procedure btCopyClick(Sender: TObject);
    procedure btImportClick(Sender: TObject);
    procedure btFactoryClick(Sender: TObject);
    procedure btExportClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  TempMessage : PChar;
  SysExMessage : Array[1..128] Of TSysExRecord;
  MainForm: TMainForm;
  ClipBoardMessage : TSysExRecord;

implementation

uses UCapture, USysExBody, UPromptName, UFactory;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
Var I : Byte;
begin
  MidiInputPort.Open;
//  MidiInputPort.Open
  MidiOutputPort.Open;
  With tbSysExList Do
  Begin
    ColWidths[0] := 30;
    ColWidths[1] := 230;
    ColWidths[2] := 30;
    ColWidths[3] := 30;
    Cells[0,0] := 'Bank';
    Cells[1,0] := 'Name';
    Cells[2,0] := 'Size';
    Cells[3,0] := 'Auto';
  End; {for}
  For I := 1 To 127 Do DeleteRow(I);
end;

procedure TMainForm.FormDeactivate(Sender: TObject);
begin
  MidiInputPort.Close;
  MidiOutputPort.Close;
end;

procedure TMainForm.btCaptureClick(Sender: TObject);
begin
  MidiInputPort.Start; {Start receiving SysEx-Data}
  frmSysExBody.SysBody.Lines.Clear; {Clears the TMemo SysEx-Body}
  frmReceiving.ShowModal; {Show the "Waiting for message" Window }
end;

procedure TMainForm.MidiInputPortMidiInput(Sender: TObject);
var ThisEvent        : TMyMidiEvent;
    TempSysEx : TSysExBody;
begin
  with (Sender As TMidiInput) do
  begin
    while (MessageCount > 0) do
    begin
      { Get the event as an object }
      thisEvent := GetMidiEvent;
      { Close the "Waiting for message" Window }
      frmReceiving.Close;
      { Stop recording }
      MidiInputPort.Stop;
      With tbSysExList Do
      Begin
        { Saves the captured message into our array }
        SysExToChar(ThisEvent.Sysex, ThisEvent.SysexLength, TempSysEx);
        frmSysExBody.SysBody.Lines.Add(SysExToString(TempSysEx, ThisEvent.SysexLength));
        { Show the captured message into a TMemo }
        If frmSysExBody.ShowModal = mrOk Then
        Begin
          SysExToChar(ThisEvent.Sysex, ThisEvent.SysexLength, SysExMessage[Row].SysExBody);
          SysExMessage[Row].Size := ThisEvent.SysexLength;
          UpdateRow(Row);
        End;
      End;
      { Event was dynamically created by GetMyMidiEvent so must	free it here }
      thisEvent.Free;
    end; {while}
  end; {with}
end;

procedure TMainForm.btNameClick(Sender: TObject);
begin
  With tbSysExList Do
  Begin
    if Cells[1, Row] = '<free>' Then
      frmPromptName.edSysExName.Text := ''
    Else frmPromptName.edSysExName.Text := Cells[1, Row];
    If frmPromptName.ShowModal = mrOk Then
    Begin
       Cells[1, Row] := frmPromptName.edSysExName.Text;
       SysExMessage[Row].MsgName := frmPromptName.edSysExName.Text;
    End;
  End;
end;

procedure TMainForm.btAutoClick(Sender: TObject);
begin
  With tbSysExList Do
  Begin
    If SysExMessage[Row].Size > 0 Then
    Begin
      SysExMessage[Row].Auto := Not SysExMessage[Row].Auto;
      if SysExMessage[Row].Auto Then Cells[3, Row] := 'yes'
                                Else Cells[3, Row] := 'no';
    End;                            
  End;
end;

procedure TMainForm.btEditClick(Sender: TObject);
Var TempStr : String;
    I       : Byte;
    TempMsg : TSysExRecord;
begin
  With tbSysExList Do
  Begin
    frmSysExBody.SysBody.Lines.Clear;
    If SysExMessage[Row].Size = 0 Then SysExMessage[Row].MsgName := ''
    Else
      frmSysExBody.SysBody.Lines.Add(SysExToString(SysExMessage[Row].SysExBody,
        SysExMessage[Row].Size));
    TempMsg := SysExMessage[Row];
    If frmSysExBody.ShowModal = mrOk Then
    Begin
      TempStr := '';
      With frmSysExBody.SysBody.Lines Do
        For I := 0 To Count - 1 Do
          TempStr := TempStr + Strings[I];
      SysExMessage[Row] := StringToSysEx(TempStr);
      SysExMessage[Row].MsgName := TempMsg.MsgName;
      SysExMessage[Row].Auto    := TempMsg.Auto;
      UpdateRow(Row);
    End;
  End;
end;

procedure TMainForm.DeleteRow(RowNr : Byte);
Var I : Byte;
Begin
  With tbSysExList Do
  Begin
    Cells[0, RowNr] := IntToStr(RowNr);
    Cells[1, RowNr] := '<free>';
    Cells[2, RowNr] := '0';
    Cells[3, RowNr] := 'no';
  End;
  With SysExMessage[RowNr] Do
  Begin
    For I := 0 To 200 Do SysExBody[I] := #0;
    MsgName := '';
    Size := 0;
    Auto := False;
  End; {with}
End;

procedure TMainForm.UpdateRow(RowNr : Byte);
Begin
  With SysExMessage[RowNr] Do
  Begin
    If Size > 0 Then
    Begin
      With tbSysExList Do
      Begin
        Cells[0, RowNr] := IntToStr(RowNr);
        Cells[1, RowNr] := MsgName;
        Cells[2, RowNr] := IntToStr(Size);
        If Auto Then
          Cells[3, RowNr] := 'yes'
        Else
          Cells[3, RowNr] := 'no';
      End; {With}
    End; {If}
  End; {with}
End;


procedure TMainForm.btSendClick(Sender: TObject);
var  sMidi : String;
begin
  { Send the current message only if the size is greater than zero }
  If SysExMessage[tbSysExList.Row].Size > 0 Then
  Begin
    sMidi := SysExToSend(SysExMessage[tbSysExList.Row].SysExBody,
      SysExMessage[tbSysExList.Row].Size);
    MidiOutputPort.PutLong(Pointer(sMidi), Length(sMidi));
  End;
end;

procedure TMainForm.btCutClick(Sender: TObject);
begin
  if MessageDlg('Delete the Current Message?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
  Begin
    ClipBoardMessage := SysExMessage[tbSysExList.Row];
    DeleteRow(tbSysExList.Row);
  End;
end;

procedure TMainForm.btSendAllClick(Sender: TObject);
var  sMidi    : String;
     I, Count : Byte;
begin
  Count := 0;
  For I := 1 To 127 Do If SysExMessage[I].Size > 0 Then Inc(Count);
  gProgress.MaxValue := Count;
  For I := 1 To 127 Do
  Begin
    { Send the current message only if the size is greater than zero }
    If SysExMessage[I].Size > 0 Then
    Begin
      gProgress.Progress := I;
      Delay(150);
      sMidi := SysExToSend(SysExMessage[I].SysExBody, SysExMessage[I].Size);
      MidiOutputPort.PutLong(Pointer(sMidi), Length(sMidi));
    End;
  End;
  gProgress.Progress := 0;
end;

procedure TMainForm.btPasteClick(Sender: TObject);
begin
  If ClipBoardMessage.Size > 0 Then
  Begin
    If SysExMessage[tbSysExList.Row].Size = 0 Then
      SysExMessage[tbSysExList.Row] := ClipBoardMessage
    Else
      if MessageDlg('Overwrite the Current Message?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        SysExMessage[tbSysExList.Row] := ClipBoardMessage;
    UpdateRow(tbSysExList.Row);
  End;  
end;

procedure TMainForm.btCopyClick(Sender: TObject);
begin
  ClipBoardMessage := SysExMessage[tbSysExList.Row];
end;

procedure TMainForm.btImportClick(Sender: TObject);
Var F        : File Of Byte;
    Cnt      : Byte;
    SysExMsg : TSysExBody;
    TempStr  : String;
    CanDo    : Boolean;
begin
  CanDo := False;
  If SysExMessage[tbSysExList.Row].Size = 0 Then CanDo := True
  Else
    if MessageDlg('Overwrite the Current Message?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      CanDo := True;
  If CanDo Then
  Begin
    Cnt := 0;
    TempStr := '';
    If OpenDialog.Execute Then
    Begin
      AssignFile(F, OpenDialog.FileName);
      Reset(F);
      While Not Eof(F) Do
      Begin
        Read(F, Byte(SysExMsg[Cnt]));
        Inc(Cnt);
      End;
      CloseFile(F);
      SysExMessage[tbSysExList.Row].SysExBody := SysExMsg;
      SysExMessage[tbSysExList.Row].Size      := Cnt;
      SysExMessage[tbSysExList.Row].MsgName   := ExtractFileName(OpenDialog.FileName);
      UpdateRow(tbSysExList.Row);
    End;
  End;
end;

procedure TMainForm.btFactoryClick(Sender: TObject);
Var CanDo    : Boolean;
    TempStr  : String;
    I        : Integer;
begin
  CanDo := False;
  If SysExMessage[tbSysExList.Row].Size = 0 Then CanDo := True
  Else
    if MessageDlg('Overwrite the Current Message?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      CanDo := True;
  If CanDo Then
  Begin
    If frmFactory.ShowModal = mrOk Then
    Begin
      If frmSysExBody.SysBody.Lines.Count > 0 Then
      If frmSysExBody.ShowModal = mrOk Then
      Begin
        TempStr := '';
        With frmSysExBody.SysBody.Lines Do
          For I := 0 To Count - 1 Do
            TempStr := TempStr + Strings[I];
        SysExMessage[tbSysExList.Row] := StringToSysEx(TempStr);
        SysExMessage[tbSysExList.Row].MsgName := '';
        SysExMessage[tbSysExList.Row].Auto    := True;
        UpdateRow(tbSysExList.Row);
      End;
    End;
  End;
end;

procedure TMainForm.btExportClick(Sender: TObject);
Var F        : File Of Byte;
    Cnt      : Byte;
    SysExMsg : TSysExRecord;
begin
  SysExMsg := SysExMessage[tbSysExList.Row];
  If SysExMsg.Size <> 0 Then
  Begin
    If SysExMessage[tbSysExList.Row].MsgName <> '' Then
      SaveDialog.FileName := CheckFileName(SysExMessage[tbSysExList.Row].MsgName);
    If SaveDialog.Execute Then
    Begin
      AssignFile(F, SaveDialog.FileName);
      Rewrite(F);
      For Cnt := 0 To SysExMsg.Size - 1 Do Write(F, Byte(SysExMsg.SysExBody[Cnt]));
      CloseFile(F);
    End;
  End;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  MidiInputPort.Close;
  MidiOutputPort.Close;
end;

end.
