unit USysXGlob;

interface

Uses SysUtils, Windows, Forms, UTiFTracker;

const SysExSize = 4096;

type
  TSysExBody   = Array[0..SysExSize] Of Char;

  TSysExRecord = Record
                   SysExBody    : TSysExBody;
                   MsgName      : String[30];
                   Size         : Word;
                   Auto         : Boolean;
                 End;

  TSysEx       = Record
                   SyxName,SyxMsg : String;
                 End;

  TValue       = String[2];

  TChannelPar = Array[0..1] Of TValue;

  TIllegalChars = Array[0..8] Of Char;
const

  GSReset = '00';
  GSExit  = '7F';


  IllegalChars   : TIllegalChars = ('$', ':', '/', '\', '"', '*', '$', '%', '&');
  ChannelType    : TChannelPar = ('00', '01');
  ChanAfterTouch : TChannelPar = ('7F', '40');

  BaseSysExBody = 'F041104212';

/////////////////////// R E V E R B   M E S S A G E S //////////////////////////

  SysExRevMacro     : TSysEx = (SyxName : 'Reverb Macro';
                                SyxMsg  : '400130');

  SysExRevCharacter : TSysEx = (SyxName : 'Reverb Character';
                                SyxMsg  : '400131');

  SysExRevPreLPF    : TSysEx = (SyxName : 'Reverb Pre-LPF';
                                SyxMsg  : '400132');

  SysExRevLevel     : TSysEx = (SyxName : 'Reverb Level';
                                SyxMsg  : '400133');

  SysExRevTime      : TSysEx = (SyxName : 'Reverb Time';
                                SyxMsg  : '400134');

  SysExRevDelay     : TSysEx = (SyxName : 'Reverb Delay Feedback';
                                SyxMsg  : '400135');

  SysExRevSend      : TSysEx = (SyxName : 'Reverb Send Level To Chorus';
                                SyxMsg  : '400136');

/////////////////////// C H O R U S   M E S S A G E S //////////////////////////

  SysExChorMacro    : TSysEx = (SyxName : 'Chorus Macro';
                                SyxMsg  : '400138');

  SysExChorPreLPF   : TSysEx = (SyxName : 'Chorus Pre-LPF';
                                SyxMsg  : '400139');

  SysExChorLevel    : TSysEx = (SyxName : 'Chorus Level';
                                SyxMsg  : '40013A');

  SysExChorFeedBack : TSysEx = (SyxName : 'Chorus FeedBack';
                                SyxMsg  : '40013B');

  SysExChorDelay    : TSysEx = (SyxName : 'Chorus Delay';
                                SyxMsg  : '40013C');

  SysExChorRate     : TSysEx = (SyxName : 'Chorus Rate';
                                SyxMsg  : '40013D');

  SysExChorDepth    : TSysEx = (SyxName : 'Chorus Depth';
                                SyxMsg  : '40013E');

  SysExChorSend     : TSysEx = (SyxName : 'Chorus Send Level To Reverb';
                                SyxMsg  : '40013F');

////////////////// V O I C E   R E S E R V E R V A T I O N /////////////////////

  SysExVoiceReserve : TSysEx = (SyxName : 'Voice Reserve';
                                SyxMsg  : '400110');

///////////////////// S Y S T E M   P A R A M E T E R S ////////////////////////

  SysExMasterVolume : TSysEx = (SyxName : 'Master Volume';
                                SyxMsg  : '400004');

  SysExMstKeyShift  : TSysEx = (SyxName : 'Master Key Shift';
                                SyxMsg  : '400005');

  SysExMasterPan    : TSysEx = (SyxName : 'Master Pan';
                                SyxMsg  : '400006');

  SysExGSMode       : TSysEx = (SyxName : 'GS Mode';
                                SyxMsg  : '40007F');

///////////////////// S Y S T E M   P A R A M E T E R S ////////////////////////

  SysExUseForRhythm : TSysEx = (SyxName : 'Use For Rhythm Part';
                                SyxMsg  : '401n15');

  SysExCAfTouch     : TSysEx = (SyxName : 'Channel Aftertouch On/Off';
                                SyxMsg  : '402n21');

function SysExToSend(SysExBody : TSysExBody; Size : Word) : String;
function FormatString(Str : String) : String;
procedure SysExToChar( bin: PChar; binSize: Word ; var SysExBody : TSysExBody);
function SysExToString(SysExBody : TSysExBody; Size : Word) : String;
function StringToSysEx(SysExMsg : String) : TSysExRecord;
Function CalculateChecksum(Body : String) : TValue;
function InsertChannel(MsgBody : String; Channel : Byte) : String;
Function CheckFileName(FName : String) : String;
function DeleteSpaces(Str : String) : String;

implementation

{Deletes all ilegal characters}
Function CheckFileName(FName : String) : String;
Var I : Byte;
Begin
  For I := Low(TIllegalChars) To High(TIllegalChars) Do
    While Pos(IllegalChars[I], FName) > 0 Do
      Delete(FName, Pos(IllegalChars[I], FName), 1);
  Result := FName;
End;

function FormatString(Str : String) : String;
Var I, J : Integer;
Begin
  I := 3;
  J := Length(Str);
  While I < J Do
  Begin
    Insert(' ', Str, I);
    J := Length(Str);
    Inc(I,3);
  End;
  Result := Str;
End;

procedure SysExToChar( bin: PChar; binSize: Word ; var SysExBody : TSysExBody);
var ctr: Word;
begin
  if binSize > SysExSize then binSize := SysExSize;
  for ctr := 0 to binSize-1 do
  begin
    SysExBody[Ctr] := bin^;
    Inc(Bin);
  end;
end;

function SysExToString(SysExBody : TSysExBody; Size : Word) : String;
var ctr      : Word;
    thisChar : Char;
Begin
  Result := '';
  if Size > SysExSize then Size := SysExSize;
  for ctr := 0 to Size - 1 do
  begin
    thisChar := SysExBody[Ctr];
    Result := Result + Format('%2.2x ', [Integer(thisChar)]);
  end;
End;

function DeleteSpaces(Str : String) : String;
Begin
  While Pos(' ', Str) > 0 do Delete(Str, Pos(' ', Str), 1);
  Result := Str;
End;

function StringToSysEx(SysExMsg : String) : TSysExRecord;
var ctr, I   : Word;
    TempInt  : Integer;
    TempStr  : String[2];
Begin
  Ctr := 1;
  While Pos(' ', SysExMsg) > 0 do
  Begin
    Delete(SysExMsg, Pos(' ', SysExMsg), 1);
  End;
  Result.Size := (Length(SysExMsg) Div 2);
  for I := 0 to Result.Size - 1 do
  begin
    TempStr := Copy(SysExMsg, Ctr, 2);
    TempInt := HexToInt(TempStr);
    Result.SysExBody[I] := Char(TempInt);
    Inc(Ctr,2);
  end;
End;

function SysExToSend(SysExBody : TSysExBody; Size : Word) : String;
var ctr      : Word;
    thisChar : Char;
Begin
  if Size > SysExSize then Size := SysExSize;
  for ctr := 0 to Size-1 do
  begin
    thisChar := SysExBody[Ctr];
    Result := Result + Char(ThisChar);
  end;
End;

Function CalculateChecksum(Body : String) : TValue;
{ Formula:

  General case: $80 - Sum(Data) = CheckSum
  When Sum(Data) > $80 Then substract $80 from Sum(Data) until Sum(Data) < $80
  Then take the general case to continue.
  Example : Data: $40 $00 $04 $64

  Step 1) $40 + $00 + $04 + $64 = $A8

  Step 2) $A8 > $80
          $A8 - $80 = $28

  Step 3) $80 - $28 = $58

  Checksum = $58
  ==============
}

Var SumData, Position : Integer;
Begin
  SumData := 0;
  Position := 1;
  While Position < Length(Body) Do
  Begin
    SumData := SumData + HexToInt(Copy(Body, Position, 2));
    Inc(Position, 2);
  End;
  While SumData > 128 Do SumData := SumData - 128;
  Result := IntToHex(128 - SumData, 2);
End;

function InsertChannel(MsgBody : String; Channel : Byte) : String;
{Replaces the 'n' in a Message in a valid channel number
 Example:
   MsgBody : InsertChannel('402n21', 6)
   Returns : '402621' }
Var Position : Integer;
Begin
  Case Channel Of
    10     : Channel := 0;
    11..16 : Channel := Channel - 1;
  End;
  Position := Pos('n', MsgBody);
  If Position > 0 Then
  Begin
    Delete(MsgBody, Position, 1);
    Insert(IntToHex(Channel,1), MsgBody, Position);
  End;
  Result := MsgBody;
End;

end.
